package ic2.core.block.machine.tileentity;

package ..deTemp.1746274182792;

import ic2.api.Direction;
import ic2.api.network.INetworkTileEntityEventListener;
import ic2.api.tile.IEnergyStorage;
import ic2.core.IC2;
import ic2.core.Ic2Items;
import ic2.core.audio.AudioPosition;
import ic2.core.audio.AudioSource;
import ic2.core.audio.PositionSpec;
import ic2.core.block.TileEntityBlock;
import ic2.core.init.MainConfig;
import ic2.core.network.NetworkManager;
import ic2.core.util.ConfigUtil;
import ic2.core.util.StackUtil;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.world.chunk.Chunk;

public class TileEntityTeleporter extends TileEntityBlock implements INetworkTileEntityEventListener {
  public boolean targetSet = false;
  
  public int targetX;
  
  public int targetY;
  
  public int targetZ;
  
  private AudioSource audioSource = null;
  
  private static final int EventTeleport = 0;
  
  public void readFromNBT(NBTTagCompound nbttagcompound) {
    super.readFromNBT(nbttagcompound);
    this.targetSet = nbttagcompound.getBoolean("targetSet");
    this.targetX = nbttagcompound.getInteger("targetX");
    this.targetY = nbttagcompound.getInteger("targetY");
    this.targetZ = nbttagcompound.getInteger("targetZ");
  }
  
  public void writeToNBT(NBTTagCompound nbttagcompound) {
    super.writeToNBT(nbttagcompound);
    nbttagcompound.setBoolean("targetSet", this.targetSet);
    nbttagcompound.setInteger("targetX", this.targetX);
    nbttagcompound.setInteger("targetY", this.targetY);
    nbttagcompound.setInteger("targetZ", this.targetZ);
  }
  
  protected void updateEntityServer() {
    super.updateEntityServer();
    if (this.worldObj.isBlockIndirectlyGettingPowered(this.xCoord, this.yCoord, this.zCoord) && this.targetSet) {
      boolean prevWorldChunkLoadOverride = this.worldObj.findingSpawnPoint;
      this.worldObj.findingSpawnPoint = true;
      Chunk chunk = this.worldObj.getChunkProvider().provideChunk(this.targetX >> 4, this.targetZ >> 4);
      this.worldObj.findingSpawnPoint = prevWorldChunkLoadOverride;
      if (chunk != null && StackUtil.equals(chunk.getBlock(this.targetX & 0xF, this.targetY, this.targetZ & 0xF), Ic2Items.teleporter) && chunk.getBlockMetadata(this.targetX & 0xF, this.targetY, this.targetZ & 0xF) == Ic2Items.teleporter.getItemDamage()) {
        setActive(true);
        List<Entity> entitiesNearby = this.worldObj.getEntitiesWithinAABB(Entity.class, AxisAlignedBB.getBoundingBox((this.xCoord - 1), this.yCoord, (this.zCoord - 1), (this.xCoord + 2), (this.yCoord + 3), (this.zCoord + 2)));
        if (!entitiesNearby.isEmpty()) {
          double minDistanceSquared = Double.MAX_VALUE;
          Entity closestEntity = null;
          for (Entity entity : entitiesNearby) {
            if (entity.ridingEntity == null) {
              double distSquared = (this.xCoord - entity.posX) * (this.xCoord - entity.posX) + ((this.yCoord + 1) - entity.posY) * ((this.yCoord + 1) - entity.posY) + (this.zCoord - entity.posZ) * (this.zCoord - entity.posZ);
              if (distSquared < minDistanceSquared && !(entity instanceof net.minecraft.entity.monster.EntityPigZombie)) {
                minDistanceSquared = distSquared;
                closestEntity = entity;
              } 
            } 
          } 
          teleport(closestEntity);
        } 
      } else {
        this.targetSet = false;
        setActive(false);
      } 
    } else {
      setActive(false);
    } 
  }
  
  protected void updateEntityClient() {
    super.updateEntityClient();
    if (getActive())
      spawnBlueParticles(2, this.xCoord, this.yCoord, this.zCoord); 
  }
  
  public void onUnloaded() {
    if (IC2.platform.isRendering() && this.audioSource != null) {
      IC2.audioManager.removeSources(this);
      this.audioSource = null;
    } 
    super.onUnloaded();
  }
  
  public void teleport(Entity user) {
    double distance = Math.sqrt(((this.xCoord - this.targetX) * (this.xCoord - this.targetX) + (this.yCoord - this.targetY) * (this.yCoord - this.targetY) + (this.zCoord - this.targetZ) * (this.zCoord - this.targetZ)));
    int weight = getWeightOf(user);
    if (weight != 0) {
      int energyCost = (int)(weight * Math.pow(distance + 10.0D, 0.7D) * 5.0D);
      if (energyCost <= getAvailableEnergy()) {
        consumeEnergy(energyCost);
        if (user instanceof EntityPlayerMP) {
          ((EntityPlayerMP)user).setPositionAndUpdate(this.targetX + 0.5D, this.targetY + 1.5D + user.getYOffset(), this.targetZ + 0.5D);
        } else {
          user.setPositionAndRotation(this.targetX + 0.5D, this.targetY + 1.5D + user.getYOffset(), this.targetZ + 0.5D, user.rotationYaw, user.rotationPitch);
        } 
        ((NetworkManager)IC2.network.get()).initiateTileEntityEvent((TileEntity)this, 0, true);
        if (user instanceof EntityPlayer && distance >= 1000.0D)
          IC2.achievements.issueAchievement((EntityPlayer)user, "teleportFarAway"); 
      } 
    } 
  }
  
  public void spawnBlueParticles(int n, int x, int y, int z) {
    for (int i = 0; i < n; i++) {
      this.worldObj.spawnParticle("reddust", (x + this.worldObj.rand.nextFloat()), ((y + 1) + this.worldObj.rand.nextFloat()), (z + this.worldObj.rand.nextFloat()), -1.0D, 0.0D, 1.0D);
      this.worldObj.spawnParticle("reddust", (x + this.worldObj.rand.nextFloat()), ((y + 2) + this.worldObj.rand.nextFloat()), (z + this.worldObj.rand.nextFloat()), -1.0D, 0.0D, 1.0D);
    } 
  }
  
  public void consumeEnergy(int energy) {
    List<IEnergyStorage> energySources = new LinkedList<>();
    Direction[] var3 = Direction.directions;
    int var4 = var3.length;
    for (int var5 = 0; var5 < var4; var5++) {
      Direction direction = var3[var5];
      TileEntity target = direction.applyToTileEntity((TileEntity)this);
      if (target instanceof IEnergyStorage) {
        IEnergyStorage energySource = (IEnergyStorage)target;
        if (energySource.isTeleporterCompatible(direction.getInverse().toForgeDirection()) && energySource.getStored() > 0)
          energySources.add(energySource); 
      } 
    } 
    while (energy > 0) {
      int drain = (energy + energySources.size() - 1) / energySources.size();
      Iterator<IEnergyStorage> it = energySources.iterator();
      while (it.hasNext()) {
        IEnergyStorage energySource = it.next();
        if (drain > energy)
          drain = energy; 
        if (energySource.getStored() <= drain) {
          energy -= energySource.getStored();
          energySource.setStored(0);
          it.remove();
          continue;
        } 
        energy -= drain;
        energySource.addEnergy(-drain);
      } 
    } 
  }
  
  public int getAvailableEnergy() {
    int energy = 0;
    Direction[] var2 = Direction.directions;
    int var3 = var2.length;
    for (int var4 = 0; var4 < var3; var4++) {
      Direction direction = var2[var4];
      TileEntity target = direction.applyToTileEntity((TileEntity)this);
      if (target instanceof IEnergyStorage) {
        IEnergyStorage storage = (IEnergyStorage)target;
        if (storage.isTeleporterCompatible(direction.getInverse().toForgeDirection()))
          energy += storage.getStored(); 
      } 
    } 
    return energy;
  }
  
  public int getWeightOf(Entity user) {
    boolean teleporterUseInventoryWeight = ConfigUtil.getBool(MainConfig.get(), "balance/teleporterUseInventoryWeight");
    int weight = 0;
    for (Entity ce = user; ce != null; ce = ce.riddenByEntity) {
      if (ce instanceof EntityItem) {
        ItemStack is = ((EntityItem)ce).getEntityItem();
        weight += 100 * is.stackSize / is.getMaxStackSize();
      } else if (!(ce instanceof net.minecraft.entity.passive.EntityAnimal) && !(ce instanceof net.minecraft.entity.item.EntityMinecart) && !(ce instanceof net.minecraft.entity.item.EntityBoat)) {
        if (ce instanceof EntityPlayer) {
          weight += 1000;
          if (teleporterUseInventoryWeight) {
            InventoryPlayer inv = ((EntityPlayer)ce).inventory;
            for (int i = 0; i < inv.mainInventory.length; i++) {
              if (inv.mainInventory[i] != null)
                weight += 100 * (inv.mainInventory[i]).stackSize / inv.mainInventory[i].getMaxStackSize(); 
            } 
          } 
        } else if (ce instanceof net.minecraft.entity.monster.EntityGhast) {
          weight += 2500;
        } else if (ce instanceof net.minecraft.entity.boss.EntityDragon) {
          weight += 10000;
        } else if (ce instanceof net.minecraft.entity.EntityCreature) {
          weight += 500;
        } 
      } else {
        weight += 100;
      } 
      if (teleporterUseInventoryWeight && ce instanceof EntityLivingBase) {
        EntityLivingBase living = (EntityLivingBase)ce;
        for (int i = (ce instanceof EntityPlayer) ? 1 : 0; i <= 4; i++) {
          ItemStack item = living.getEquipmentInSlot(i);
          if (item != null)
            weight += 100 * item.stackSize / item.getMaxStackSize(); 
        } 
      } 
    } 
    return weight;
  }
  
  public void setTarget(int x, int y, int z) {
    this.targetSet = true;
    this.targetX = x;
    this.targetY = y;
    this.targetZ = z;
    ((NetworkManager)IC2.network.get()).updateTileEntityField((TileEntity)this, "targetX");
    ((NetworkManager)IC2.network.get()).updateTileEntityField((TileEntity)this, "targetY");
    ((NetworkManager)IC2.network.get()).updateTileEntityField((TileEntity)this, "targetZ");
  }
  
  public List<String> getNetworkedFields() {
    List<String> ret = new Vector<>(3);
    ret.add("targetX");
    ret.add("targetY");
    ret.add("targetZ");
    ret.addAll(super.getNetworkedFields());
    return ret;
  }
  
  public void onNetworkUpdate(String field) {
    if (field.equals("active") && this.prevActive != getActive()) {
      if (this.audioSource == null)
        this.audioSource = IC2.audioManager.createSource(this, PositionSpec.Center, "Machines/Teleporter/TeleChargedLoop.ogg", true, false, IC2.audioManager.getDefaultVolume()); 
      if (getActive()) {
        if (this.audioSource != null)
          this.audioSource.play(); 
      } else if (this.audioSource != null) {
        this.audioSource.stop();
      } 
    } 
    super.onNetworkUpdate(field);
  }
  
  public void onNetworkEvent(int event) {
    switch (event) {
      case 0:
        IC2.audioManager.playOnce(this, PositionSpec.Center, "Machines/Teleporter/TeleUse.ogg", true, IC2.audioManager.getDefaultVolume());
        IC2.audioManager.playOnce(new AudioPosition(this.worldObj, this.targetX + 0.5F, this.targetY + 0.5F, this.targetZ + 0.5F), PositionSpec.Center, "Machines/Teleporter/TeleUse.ogg", true, IC2.audioManager.getDefaultVolume());
        spawnBlueParticles(20, this.xCoord, this.yCoord, this.zCoord);
        spawnBlueParticles(20, this.targetX, this.targetY, this.targetZ);
        return;
    } 
    IC2.platform.displayError("An unknown event type was received over multiplayer.\nThis could happen due to corrupted data or a bug.\n\n(Technical information: event ID " + event + ", tile entity below)\nT: " + this + " (" + this.xCoord + ", " + this.yCoord + ", " + this.zCoord + ")", new Object[0]);
  }
}
